#  TMC_seg092_AgeClass4Perc.R

#  (c) wwosniok@math.uni-bremen.de

#  Truncated minimum chi-square estimation

#  Organize percentile plots of  measurand percentile vs age, by sex
#  Data subsets are sex groups, 
#  Age group limits are either 
#  - fixed by the user (age.clss.style="fix") or 
#  - derived from data, based on minimum stratum size = x.n.min, set in 030

#  CHANGE HISTORY
#  02.01.2023 Plot files renamed
#  27.12.2022 Start
# ==============================================================================

# ===========================================================================
if (print.log.message) { cat("%%%   TMC_seg092_AgeClass4Perc  Start\n") }
# ==========================================================================

  #  Colors for percentile plot are defined in seg030
  collist.pepl <- rainbow(probs.pepl.n)

  source("TMC_seg091_OpenWindows.R")

  #  Loop over sex codes

  for (isex in 1:sex.val.n)
  { subset.s <- dataset[ ,spalte.s] == sex.val[isex]
    sexlabel <- sex.val[isex]

    if (age.class.style == "fix" & plot.fig092.010 & plot.fig092.020)
    { 
      #  Age classes were defined by user. May be overlapping.
      age.class.n <- nrow(age.class)

      cat("\n Fixed age classes, ",  sexlabel, "\n")
      print(age.class)
      # print(age.limits)
    }

    if (age.class.style == "dyn" & plot.fig092.030 & plot.fig092.040)
    { 
      #  Find age classes of size x.n.min in this sex group
      #  Age classes identified by DynAgeLimits do not overlap

      age.class <- DynAgeLimits(dataset[subset.s, spalte.a], x.n.min)
      age.class.n <- nrow(age.class)

      cat("\n Dynamic age classes, x.n.min =", x.n.min, sexlabel, "\n")
      print(age.class)
      # print(age.limits)
    }

    if (age.class.n > 1)
    {
      agelabel.vec <- rep(NA, times=age.class.n)

      for (iage.class in 1:age.class.n)
      { #  Age classes
        agelabel.vec[iage.class] <- paste(age.class[iage.class, "lo"],
                                      age.class[iage.class, "hi"],sep="-")
      }
    } else
    { # Only 1 age class 
      agelabel <- "All"
    } 

    if (age.class.n > 1)
    {
      #  Extract all data for this sex group
      x.sub    <- dataset[subset.s, spalte.w]
      x.sub.n  <- length(x.sub)

      if (!is.na(spalte.a)) { age.sub <- dataset[subset.s, spalte.a] }

      # Percentile plot for x, Sex = Ausprgung sex.val[isex]
 
      # .....................................................................
      title <- paste("Percentile plot of", xlabel, " sex =", sexlabel) 
      subtitle <- paste(date(), " / " ,outname, "_", sex.val[isex],
                            "  /  n = ", x.sub.n, sep="")

      # .....................................................................
      if (sex.val[isex] == "F" & age.class.style=="fix") 
      { 
        dev.set(fig092.010)
        MP <- PlotPercentile(fig092.010, fig092.010.file, "bmp",
                           age.sub, age.class, age.class.n,  
                           xlabel, title, subtitle,
                           x.sub, 
                           x.clip.min, x.clip.max, x.clip.by1, x.clip.by2, 
                           age.clip.min, age.clip.max, age.clip.by1, 
                           age.clip.by2, 
                           probs.pepl, probs.pepl.n, collist.pepl,
                           par.las, par.tcl)
      }   # sex.val[isex] == "F" & age.class.style=="fix"

      # .....................................................................
      if (sex.val[isex] == "F" & age.class.style=="dyn") 
      { 
        dev.set(fig092.030) 
        MP <- PlotPercentile(fig092.030, fig092.030.file, "bmp",
                           age.sub, age.class, age.class.n,  
                           xlabel, title, subtitle,
                           x.sub, 
                           x.clip.min, x.clip.max, x.clip.by1, x.clip.by2, 
                           age.clip.min, age.clip.max, age.clip.by1, 
                           age.clip.by2, 
                           probs.pepl, probs.pepl.n, collist.pepl,
                           par.las, par.tcl)
      }   # sex.val[isex] == "F" & age.class.style=="dyn"

      # .....................................................................
      if (sex.val[isex] == "M" & age.class.style=="fix") 
      { 
        dev.set(fig092.020) 
        MP <- PlotPercentile(fig092.020, fig092.020.file, "bmp",
                           age.sub, age.class, age.class.n,  
                           xlabel, title, subtitle,
                           x.sub, 
                           x.clip.min, x.clip.max, x.clip.by1, x.clip.by2, 
                           age.clip.min, age.clip.max, age.clip.by1, 
                           age.clip.by2, 
                           probs.pepl, probs.pepl.n, collist.pepl,
                           par.las, par.tcl)
      }   # sex.val[isex] == "M" & age.class.style=="fix"

      # .....................................................................
      if (sex.val[isex] == "M" & age.class.style=="dyn") 
      { 
        dev.set(fig092.040) 
        MP <- PlotPercentile(fig092.040, fig092.040.file, "bmp",
                           age.sub, age.class, age.class.n,  
                           xlabel, title, subtitle,
                           x.sub, 
                           x.clip.min, x.clip.max, x.clip.by1, x.clip.by2, 
                           age.clip.min, age.clip.max, age.clip.by1, 
                           age.clip.by2, 
                           probs.pepl, probs.pepl.n, collist.pepl,
                           par.las, par.tcl)
      }   # sex.val[isex] == "M" & age.class.style=="dyn"

    }     #  age.class.n > 1 ..
  }       #  for (isex ...

# ===========================================================================
if (print.log.message) { cat("%%%   TMC_seg092_AgeClass4Perc  End\n") }
# ==========================================================================


