###-  drift  to analyse drift effects                                                     -###
###-  Written by Farhad Arzideh (Ar)                                                      -###
###-  other authors: Bernd Wolters (Wo)                                                   -###
###                                                                                        ###
###   01.08.2014 (Wo): graphics format optional, comments and cat included                 ###
###   12.01.2015 (Ar): error messages are included                                         ###
###   12.01.2015 (Ar): columns for sex and station are redefined; e.g. all rows with       ###
###                    entries "M" or "M " or "M  " transfered to "M" and thereby imported.###
###   12.01.2015 (Ar): drift1 modificated, so that                                         ###
###                    it is not necessary to have columns of month and year of test.      ###
###                    Only the date of test should be given.                              ###
###                    Notice: data set should contain only one result from each patient.  ###
###   06.05.2015 (Ar): the new program estimates a smooth-curve for median of data monthly ###
###                    (with GAM-Model) and display it with confidence interval.           ###
###                    package "mgcv" shold be installed in R.                             ###
###   17.08.2015 (Ar): the gam-test is applied, if the datasets contain results of more    ###
###                    than 12 months.                                                     ###
###   25.08.2015 (Ar): denotation of columns for sex and hospital was changed(lines 202,203)##
###   26.08.2015 (Ar): a subfolder with a special name, generated by the name of the       ###
###                    test value (measurand) + age range + date of analysis, is created   ###
###                    in the   \results-folder (if it does not exist). The evaluations    ###
###                    (results and graphs) are saved in a subfolder \step1a created       ###
###                    in this subfolder.                                                  ### 
###   26.08.2015 (Ar): If some months or years lack in the data set, the program run       ###
###                    even so.                                                            ###
###   24.09.2015 (Ar): the weight argument in gam-function (lines 517, 562, 606)           ###
###                    werde defined as double.                                            ###
###   22.07.2016 (Ar): "error message: minimum age can not be greater than its maximum!"   ###
###   25.07.2016 (Ar): A report data, prog_log.txt, will be created and saved in           ###
###                      "result"-file.                                                    ###
###   19.09.2016 (Ar): special symobols ($, &, /, etc. and empty spaces) in the dataname   ###
###                    are replaced with underline (_).                                    ###
###   12.10.2016 (Ar): a copy of data with applied limits (age, etc.)  with columns:       ###
###                    c(Age,Value,Sex,Analyse_date) is saved in the subfolder explained   ###
###                    above with name "data_step1a.csv"                                   ###  
###   12.10.2016 (Ar): decimal places +1 is used in output                                 ###
###   13.10.2016 (Wo): testunit wird mit testname auf Grafiken ausgegeben                  ###
###   19.10.2016 (Ar): i) new variables are defined to analyse the data in a               ###
###                       given date range                                                 ### 
###                    ii) a copy of data with used columns named data_step1a.csv          ###
###                        will be saved in a subdirectory of result file.                 ###
###   12.02.2017 (Wo): saving of plots changed (incl. plot_wrapper.r)                      ###
###   01.03.2017 (Wo): logfile now for every step                                          ###
###   23.07.2017 (Wo): logfile renamend to RLE_Prog1a.log                                  ###
###   07.09.2017 (Ar): error messages were modified to give more detailed informations     ###
###   07.09.2017 (Ar): steps to perform confidence intervals for estimated smoothed        ###
###                    curves for median have been modified.                               ###
###   07.09.2017 (Wo/Ar): program can be run with time limitations (date1=start date,      ###
###                    or/and date2=end date)                                              ###  
###   08.09.2017 (Wo): logfile in Subdirectory of temp data                                ### 
###   18.09.2017 (FA): permissable uncertainty of median of data-values is calculated      ###
###                    to evaluate the drift effect (optional). This can be done, if one   ###
###                    gives actual RLs used in laboratory.                                ### 
###   07.05.2018 (Wo): file_data renamed file_temp for temp subdirectory                   ###
###                    new variable: file_data for use of raw-data                         ###
###                    file_data now used for classic only                                 ###
###   02.05.2023 (FA): because of CHANGES IN R > 4.2.0 for calling if() with a condition   ###
###                    of length greater than one, lines 440 and 464 were modified.        ###
###   03.07.2023 (Ar/Wo): read.csv fileEncoding = "UTF-8" introduced, to handle            ###
###                    German Umlaute                                                      ###
##############################################################################################
rm(list=ls(all=TRUE))
#*************************#
###-  DATA locations   -###
#*************************#
############################################
############################################
###                                      ###
###   1. gives path of the par data      ###
###                                      ###
############################################
############################################
#
cat("read arguments (1.) ....\n")
#
# Arguments are read
#
args1 <- commandArgs()
ar <- which(args1 == "--args")

#
# To get always the last --args one,
#
ar1 <- ar[length(ar)]
args <- args1[(ar1+1):length(args1)]

#
# List args contains now the real arguments
#

#
# DATA locations   
#
INPUT_PAR_FILE <- args[1]
start.time <- date()
##############################################################
##############################################################
###                                                        ###
###   2. assignment of variables in R from the par-data    ###
###                                                        ###
##############################################################
##############################################################
#
#####################
try({
#####################
# LOOK FOR INPUT-PARAMETERFILE AND SET PARAMETERS
if (INPUT_PAR_FILE != "" && file.exists(INPUT_PAR_FILE)) {
    parFileReport<-1   #Fehlerbearbeitung nicht realisiert
    PARS<-read.csv(INPUT_PAR_FILE,header=FALSE,sep=";",dec=".",fileEncoding="UTF-8")

##############################################################

    cat("assingment of variabels (2.) ....\n")

##############################################################
    dataname<-as.character(PARS[1,1])                   # name of csv-datafile
    head<-as.character(PARS[2,1])                       # header
    head<-ifelse(head=="TRUE",TRUE,FALSE)
    c2<-as.character(PARS[3,1])                         # column of measurand
    c1<-as.character(PARS[4,1])                         # column of age
    c3<-as.character(PARS[5,1])                         # column of gender
    c5<-as.character(PARS[6,1])                         # column of date of test
    c6<-as.character(PARS[7,1])                         # format of date of test
    a1<-as.numeric(as.character(PARS[8,1]))             # min age
    a2<-as.numeric(as.character(PARS[9,1]))             # max age
    m<-as.character(PARS[10,1])                         # male character
    f<-as.character(PARS[11,1])                         # female character
    com<-as.numeric(as.character(PARS[12,1]))           # number of decimal places of measurand
    dec.sep<-as.character(PARS[13,1])                   # decimal separator
    quant<-as.numeric(as.character(PARS[14,1]))         # quantiles
    c4<-as.character(PARS[15,1])                        # column of hospitalized/ambulantory 
    hospit<-as.character(PARS[16,1])                    # denotation of hospitalized/ambulantory
    testvalue<-as.character(PARS[17,1])                 # description text of test value
    flag_file<-as.character(PARS[18,1])                 # result file
    #   free_variable<-as.character(PARS[19,1])         # free for other user eg OSM/swisslab 
    file_prog<-as.character(PARS[20,1])                 # subdirectory program
    file_res<-as.character(PARS[21,1])                  # subdirectory results
    file_temp<-as.character(PARS[22,1])                 # subdirectory data
    main_folder1<-as.character(PARS[23,1])              # main folder of this tool (slash)
    graph_format<-as.character(PARS[24,1])              # graphics format (bmp, jpg)
    error_file<-as.character(PARS[25,1])                # absolute path and filename of error file
    savename1a<-as.character(PARS[26,1])                 # absolute path and filename of result_drift file
    r_func<-as.character(PARS[27,1])                    # absolute path for r functions   
    testunit<-as.character(PARS[28,1])                  # unit of testvalue (for praphical output)
    #  group<-as.character(PARS[29,1])                  # defines subgroups to evaluate: n.d.
    date1<-as.character(PARS[30,1])                     # start date of data to be analysed 
    date2<-as.character(PARS[31,1])                     # end date of data to be analysed
    RL1m<-as.numeric(gsub(",",".",as.character(PARS[32,1])))            # actual used LRL for male subject 
    RL2m<-as.numeric(gsub(",",".",as.character(PARS[33,1])))            # actual used URL for male subject
    RL1w<-as.numeric(gsub(",",".",as.character(PARS[34,1])))            # actual used LRL for female subject 
    RL2w<-as.numeric(gsub(",",".",as.character(PARS[35,1])))            # actual used URL for female subject
    RL1a<-as.numeric(gsub(",",".",as.character(PARS[36,1])))            # actual used LRL for all subject 
    RL2a<-as.numeric(gsub(",",".",as.character(PARS[37,1])))            # actual used URL for all subject
    file_data<-as.character(PARS[38,1])                                 # subdirectory for data (identically with temp if standard dataset used)
    }
##############################################
par_file<-paste(main_folder1,"/",file_temp,"/", sep="")    # subdirectory of par
cat("Temppfad: ", par_file, "\n\n")
data_file<-paste(main_folder1,"/",file_data,"/", sep="")   # subdirectory of data (for classic only)
cat("Datenpfad:", data_file, "\n")
result_file<-paste(main_folder1,"/",file_res,"/", sep="")  # subdirectory of results
prog_file<-paste(main_folder1,"/",file_prog,"/", sep="")   # subdirectory of prog
##############################################
# create subfolders in folder \results to save the results and graphs.
testvalue<-gsub('([[:punct:]])|\\s','_',testvalue)  # replace special symobols ($, &, /, etc. and empty spaces) with underline (_).  
result_file0<-paste(result_file, testvalue,"age",a1,a2, "date",format(Sys.time(), "%Y-%m-%d"), sep = "_") 
dir.create(result_file0, showWarnings = TRUE)
result_file1<-paste(result_file0 , "/","step1a", sep = "")
dir.create(result_file1, showWarnings = TRUE)
##############################################
##############################################
num.sink <- sink.number()
if (num.sink > 0)  for (i in 1:num.sink)  sink() 
logdat<-paste(main_folder1,"/",file_temp,"/RLE_Prog1a.log", sep="")
})
###############################
# If R delivers error message, 
# it is sent up to the error_file.
###############################  
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: the entries in tab parameter for analysis in start dialog box are not defined, completely or correctely. Or the user has no admin permission to create subfolders"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
##############################################################
if (a1 > a2) {            
  errmess<-"error: minimum age can not be greater than its maximum!"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  } 
###############################################################
###                                                         ###
###               3. read of data and save copy             ###
###                                                         ###
###############################################################
{
if (length(PARS[1,])==1) {             # if only one data should be analysed
  file.name<-paste(data_file,dataname, sep="")
  check<-1
  suppressWarnings(data<-try(read.table(file=file.name,dec=dec.sep,header=head,sep=";"), TRUE))
  if (class(data) == "try-error") {
    errmess<-"error: either data set does not exist, or entries in parameter for analysis are not defined correctely or data in each line are not separated by ; or entries have not the same class."
    write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
    write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
    quit(save = "no", status = 0, runLast = FALSE)
    }else{
    data<-read.table(file=file.name,dec=dec.sep,header=head,sep=";")
    }
  }else{
  # if more than one data should be analysed
  dataname<-PARS[1,]
  J<-length(dataname)
  data<-NULL
  for (j in 1:J)  {
    check<-2
    dataname1<-dataname[[j]]
    file.name<-paste(data_file,dataname1, sep="")
    suppressWarnings(data1<-try(read.table(file=file.name,dec=dec.sep,header=head,sep=";"), TRUE))
    if (class(data1) == "try-error") {
      errmess<-"error: either data set does not exist, or entries in parameter for analysis are not defined correctely or data in each line are not separated by ; or entries have not the same class."
      write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
      write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
      quit(save = "no", status = 0, runLast = FALSE)
      }else{
      data1<-read.table(file=file.name,dec=dec.sep,header=head,sep=";")
      suppressWarnings(data<-try(rbind(data,data1),TRUE))
      if (class(data) == "try-error") {
        errmess<-"error: the datasets have different structure and they can not be analysed together"
        write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
        write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
        quit(save = "no", status = 0, runLast = FALSE)
        }
      }
    }
  }
}
####################################
nw<-ncol(data)
v1<-c(LETTERS[1:nw])
v2<-c(1:nw)
mat<-matrix(c(v1,v2),nrow=2,byrow=TRUE)
c1<-as.numeric(mat[,mat[1,]==c1][2])
c2<-as.numeric(mat[,mat[1,]==c2][2])
c3<-as.numeric(mat[,mat[1,]==c3][2])
c4<-as.numeric(mat[,mat[1,]==c4][2])
c5<-as.numeric(mat[,mat[1,]==c5][2])
if (!is.na(c4)) {
  data<-subset(data,subset=(data[,c4]==hospit))
  mycol<-c(c3,c4)
  for (i in mycol) {  # columns for sex and station are redefined, that
                      # causes e.g. all rows with entries "M" or "M " or "M  " transfered to "M".  
    data[,i] <- gsub(" ", "", data[,i])
    }  
  }else {
  data[,c3] <- gsub(" ", "", data[,c3])
  }
###############################################
## load of required libraries:
try({
   library(msm)
   library(geoR)
   library(mgcv)                                             # GAM-test nach Simon Wood
   })                                                        
errmess<-geterrmessage()                                             # test whether necassary packages have been loaded.
if (errmess=="") {
   errmess<-"no error"
    write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   } else {
   sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("R version is: ", as.character(getRversion()), "\n", sep = "")
   # cat("SessionInfo: ", sessionInfo(), sep="")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
cat("all libraries loaded ....\n")
###################################################
# test whether the necessary columns are defined:
if (!is.na(c1) & !is.na(c2) & !is.na(c3) & !is.na(c5) & !is.na(c6)) {
  errmess<-"no error"
  # write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  } else {
  errmess<-"error: columns of value/age/sex/date of data or format of date does not exit or are not defined correctly"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
###################################################
# Necessary functions from prog subdirectories are now loaded:
try({
  source(paste(r_func,"/data_sep1.R",sep=""))
  source(paste(r_func,"/data_gr.R",sep=""))
  source(paste(r_func,"/plot_wrapper.R",sep=""))
  source(paste(r_func,"/pU.R",sep=""))
  })
errmess<-geterrmessage() 
if (errmess!="") {                #   test whether required functions have been loaded.     
  errmess<-"error: one or more required functions to analyse the data could not be loaded. Either subdirectory R functions has not been set correctly or some functions are removed. Create R functions again."
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
cat("all functions loaded ....\n")
############################################
data<-data[,c(c1,c2,c3,c5)]
data<-na.omit(data)         # removes rows with NA values
if (nrow(data) == 0){
  errmess<-"error: columns of value/age/sex/date of data are not defined correctly or there is no complete case"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
############################################
sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
cat("================================================================\n\n")
cat("Data have been imported correctly.","\n\n")
cat("================================================================\n\n")
sink()
############################################
try({
  data[,4]<-as.Date(data[,4], format=c6) # appending format for date of analysis   
  })
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: either choosed format for date is false or column of date is not defined correctly."
  sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
###########################################
try({
  data$month<-format(data[,4],"%m") # month of data
  data$year<- format(data[,4],"%Y") # year of data
  })
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: choosed format for date is false."
  sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
################################################################
#
# 3.a if one uses start and end date to analyse of data
#  
################################################################
try({
  if(!is.na(date1)) date1<-as.Date(date1,,format=c6)
  if(!is.na(date2)) date2<-as.Date(date2,,format=c6)
  }) 
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: choosed formats for start or end date are false. You can set start and end date by NA"
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###############################################
if (!is.na(date1) & !is.na(date2) & date1 >= date2 ) {            
errmess<-"error: start date can not be greater as end date! You can set start and end date by NA"
sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
end.time <- date()
cat("Program start:", start.time,"\n")
cat("program end:  ", end.time,"\n")
cat("================================================================\n\n")
cat(errmess, "\n")
sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   } 
try({
if(!is.na(date1)) data<-subset(data,subset=(data[,4]>=date1))
   }) 
errmess<-geterrmessage() 
if (errmess!="" || dim(data)[1]==0) {            
   errmess<-"error: control whether you have choosed a correct value for start date! You can set start and end date by NA"
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }else{
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   cat("================================================================\n\n")
   cat("Data have been limited to the given start date.","\n\n")
   cat("================================================================\n\n")
   sink()
   }
###############################################
try({
if(!is.na(date2)) data<-subset(data,subset=(data[,4]<=date2))
   }) 
errmess<-geterrmessage() 
if (errmess!="" || dim(data)[1]==0) {            
   errmess<-"error: control whether you have choosed a correct value for end date! You can set start and end date by NA"
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }else{
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   cat("================================================================\n\n")
   cat("Data have been limited to the given end date.","\n\n")
   cat("================================================================\n\n")
   sink()
   }
###############################################
################################################################ 
###                                                          ###
###   3.b save copy of used data                             ### 
###                                                          ###
################################################################
result_file1a<-paste(result_file0, "/","data_step1a.csv", sep = "")
colname<-c("Age","Value","Gender","Analyse_date")
write.table(data[,1:4], file=result_file1a, quote=FALSE, row.names=FALSE,sep=";",col.names=colname)   
cat("dataset copied in subdirectory ....\n")  
sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
cat("================================================================\n\n")
cat(paste("A copy of data with used columns in defined date range (if given) was put in ", result_file1a, sep=" "),"\n\n")
cat("================================================================\n\n")
sink()
###########################################    
###########################################
cat("program is running ....\n")
if (is.na(testvalue))   {
   testvalue<-""
   }
if (!is.numeric(data[,1]) | !is.numeric(data[,2]))  {
   errmess<-"error: columns of value or age is/are not numeric. Control if the entries in each of these columns have the same format."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
try(d<-data.sep(data=data,c1=1,c2=2,c3=3,a1=a1,a2=a2,m=m,w=f))        # erweitern um u=u
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: either false given columns of values/ages/sex/date or false denotation of sex or date format or data set does not contain male and female subjects or entries have not the same class."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###########################################################################
dbm<-d$all                        # values for male+female subjects (age group: a1< age <= a2)
dmbm<-d$m                         # values for male subjects (age group: a1< age <= a2)
dwbm<-d$w                         # values for female subjects (age group: a1< age <= a2)
###########################################################################
# Date of test
try({
   dbm <- within(dbm, Date <- as.Date(paste(dbm$year, dbm$month,rep(1,length(dbm$year)),sep = "-")))
   dmbm <- within(dmbm, Date <- as.Date(paste(dmbm$year, dmbm$month,rep(1,length(dmbm$year)),sep = "-")))
   dwbm <- within(dwbm, Date <- as.Date(paste(dwbm$year, dwbm$month,rep(1,length(dwbm$year)),sep = "-")))
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: either choosed format for date is false or column of date is not defined correctly."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
if (ncol(dbm)<6 | ncol(dmbm)<6 | ncol(dwbm)<6 | nrow(dbm)==0 | nrow(dmbm)==0 | nrow(dwbm)==0){
   errmess<-"error: either false given columns of values/ages/sex/date or false denotation of sex or date format or data set does not contain male and female subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
#####################################
# Calculation of median of data, monthly:
###
med.all<-tapply(dbm[,2], dbm$Date,median)
med.male<-tapply(dmbm[,2], dmbm$Date,median)
med.female<-tapply(dwbm[,2], dwbm$Date,median)
#######################################
#########################
# Number of data monthly.
n.all<-tapply(dbm[,2], dbm$Date,length)
n.male<-tapply(dmbm[,2], dmbm$Date,length)
n.female<-tapply(dwbm[,2], dwbm$Date,length)
########################
# Definition of label for x-axis for all patients:
try({
   labelm<-format(as.Date(names(med.male)),"%b.%y")
   labelw<-format(as.Date(names(med.female)),"%b.%y")
   label1<-format(as.Date(names(med.all)),"%b.%y")                      
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: control the date format."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###########################################################################
###########################################################################
###                                                                     ###
###    4. Calculation of quantiles                                      ###
###                                                                     ###
###########################################################################
###########################################################################

cat("program is calculating quantiles ....\n")
##############################
#quantiles
q05<-tapply(dbm[,2], dbm$Date,quantile,prob=0.05)
q25<-tapply(dbm[,2], dbm$Date,quantile,prob=0.25)
q50<-tapply(dbm[,2], dbm$Date,quantile,prob=0.50)
q75<-tapply(dbm[,2], dbm$Date,quantile,prob=0.75)
q95<-tapply(dbm[,2], dbm$Date,quantile,prob=0.95)
q05m<-tapply(dmbm[,2], dmbm$Date,quantile,prob=0.05)
q25m<-tapply(dmbm[,2], dmbm$Date,quantile,prob=0.25)
q50m<-tapply(dmbm[,2], dmbm$Date,quantile,prob=0.50)
q75m<-tapply(dmbm[,2], dmbm$Date,quantile,prob=0.75)
q95m<-tapply(dmbm[,2], dmbm$Date,quantile,prob=0.95)
q05w<-tapply(dwbm[,2], dwbm$Date,quantile,prob=0.05)
q25w<-tapply(dwbm[,2], dwbm$Date,quantile,prob=0.25)
q50w<-tapply(dwbm[,2], dwbm$Date,quantile,prob=0.50)
q75w<-tapply(dwbm[,2], dwbm$Date,quantile,prob=0.75)
q95w<-tapply(dwbm[,2], dwbm$Date,quantile,prob=0.95)
monm<-length(q95m)
monw<-length(q95w)
mon1<-length(q95)
###################################
if (quant==1)  {
   ## quant=1 only 50 percentile
   cat("program is plotting 50 percentile ....\n")
   # plot male
   bild4<-paste(result_file,"Months_m.",graph_format,sep="")                 
   bild4.1<-paste(result_file1,"/","Months_m.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild4,width=7,height=5)
   plot(1:monm,q50m,type="o",col="blue",ylim=c(min(q25m),max(q75m)),xlab="Months",ylab=paste(testvalue," ",testunit),
   main="50 (blue) percentile (male)",lty=2,xaxt="n")
   axis(1, at=1:monm,labels =labelm)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild4)
   close_Plot(graph_format=graph_format,file=bild4.1)
   # plot female
   bild5<-paste(result_file,"Months_w.",graph_format,sep="")
   bild5.1<-paste(result_file1,"/","Months_w.",graph_format,sep="")   
   open_plot(graph_format=graph_format,file=bild5,width=7,height=5)
   plot(1:monw,q50w,type="o",col="blue",ylim=c(min(q25w),max(q75w)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="50 (blue) percentile for female",lty=2,xaxt="n")
   axis(1, at=1:monw,labels =labelw)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild5)
   close_plot(graph_format=graph_format,file=bild5.1)
   # plot male and female (all)
   bild6<-paste(result_file,"Months_a.",graph_format,sep="")
   bild6.1<-paste(result_file1,"/","Months_a.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild6,width=7,height=5)
   plot(1:mon1,q50,type="o",col="blue",ylim=c(min(q25),max(q75)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="50 (blue) percentile (all)",lty=2,xaxt="n")
   axis(1, at=1:mon1,labels =label1)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild6)
   close_plot(graph_format=graph_format,file=bild6.1)
###############################################################
   } else if (quant==3)  {
   # quant=3 with 25, 50 , 75 percentiles
   cat("program is plotting 25, 50 and 75 perecentile ....\n")
   # plot male
   bild4<-paste(result_file,"Months_m.",graph_format,sep="")
   bild4.1<-paste(result_file1,"/","Months_m.",graph_format,sep="")   
   open_plot(graph_format=graph_format,file=bild4,width=7,height=5)
   plot(1:monm,q50m,type="o",col="blue",ylim=c(min(q25m),max(q75m)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="25(green), 50(blue) and 75(red) percentiles (male)",lty=2,xaxt="n")
   lines(q25m,typ="o",col="green3",lty=2)
   lines(q75m,typ="o",col="red",lty=2)
   axis(1, at=1:monm,labels =labelm)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild4)
   close_plot(graph_format=graph_format,file=bild4.1)
   # plot female
   bild5<-paste(result_file,"Months_w.",graph_format,sep="")
   bild5.1<-paste(result_file1,"/","Months_w.",graph_format,sep="")     
   open_plot(graph_format=graph_format,file=bild5,width=7,height=5)
   plot(1:monw,q50w,type="o",col="blue",ylim=c(min(q25w),max(q75w)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="25(green), 50(blue) and 75(red) percentiles (female)",lty=2,xaxt="n")
   lines(q25w,typ="o",col="green3",lty=2)
   lines(q75w,typ="o",col="red",lty=2)
   axis(1, at=1:monw,labels =labelw)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild5)
   close_plot(graph_format=graph_format,file=bild5.1)
   # plot male and female (all)
   bild6<-paste(result_file,"Months_a.",graph_format,sep="")
   bild6.1<-paste(result_file1,"/","Months_a.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild6,width=7,height=5)
   plot(1:mon1,q50,type="o",col="blue",ylim=c(min(q25),max(q75)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="25(green), 50(blue) and 75(red) percentiles (all)",lty=2,xaxt="n")
   lines(q25,typ="o",col="green3",lty=2)
   lines(q75,typ="o",col="red",lty=2)
   axis(1, at=1:mon1,labels =label1)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild6)
   close_plot(graph_format=graph_format,file=bild6.1)
   } else  {
   # quant=5 with 5,25, 50, 75, 95 percentiles
   cat("program is plotting 5, 25, 50, 75 and 95 percentile ....\n")
   # plot male
   bild4<-paste(result_file,"Months_m.",graph_format,sep="")
   bild4.1<-paste(result_file1,"/","Months_m.",graph_format,sep="")   
   open_plot(graph_format=graph_format,file=bild4,width=7,height=5)
   plot(1:monm,q50m,type="o",col="blue",ylim=c(min(q05m),max(q95m)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="5(black, 25(green), 50(blue), 75(red) and 95(orange) percentiles (male)",lty=2,xaxt="n")
   lines(q05m,typ="o",col="black",lty=2)
   lines(q25m,typ="o",col="green3",lty=2)
   lines(q75m,typ="o",col="red",lty=2)
   lines(q95m,typ="o",col="orange",lty=2)
   axis(1, at=1:monm,labels =labelm)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild4)
   close_plot(graph_format=graph_format,file=bild4.1)
   # plot female
   bild5<-paste(result_file,"Months_w.",graph_format,sep="")
   bild5.1<-paste(result_file1,"/","Months_w.",graph_format,sep="")   
   open_plot(graph_format=graph_format,file=bild5,width=7,height=5)
   plot(1:monw,q50w,type="o",col="blue",ylim=c(min(q05w),max(q95w)),xlab="Months",ylab=paste(testvalue," ",testunit),
       main="5(black), 25(green), 50(blue), 75(red) and 95(orange) percentiles (female)",lty=2,xaxt="n")
   lines(q05w,typ="o",col="black",lty=2)
   lines(q25w,typ="o",col="green3",lty=2)
   lines(q75w,typ="o",col="red",lty=2)
   lines(q95w,typ="o",col="orange",lty=2)
   axis(1, at=1:monw,labels =labelw)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild5)
   close_plot(graph_format=graph_format,file=bild5.1)
   # plot male and female (all)
   bild6<-paste(result_file,"Months_a.",graph_format,sep="")
   bild6.1<-paste(result_file1,"/","Months_a.",graph_format,sep="")   
   open_plot(graph_format=graph_format,file=bild6,width=7,height=5)
   plot(1:mon1,q50,type="o",col="blue",ylim=c(min(q05),max(q95)),xlab="Months",ylab=paste(testvalue," ",testunit),
        main="5(black), 25(green), 50(blue), 75(red) and 95(orange) percentiles (all)",lty=2,xaxt="n")
   lines(q05,typ="o",col="black",lty=2)
   lines(q25,typ="o",col="green3",lty=2)
   lines(q75,typ="o",col="red",lty=2)
   lines(q95,typ="o",col="orange",lty=2)
   axis(1, at=1:mon1,labels =label1)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild6)
   close_plot(graph_format=graph_format,file=bild6.1)
   }
###############################################################
# lower and upper limits of permissable uncertainty of median of results 
# in the overall time of data gathering for male , femal and all subjects can be calculated if  
# the corresponding RL have been given:
try({
Q50.m<-quantile(dmbm[,2],prob=0.50)
Q50.w<-quantile(dwbm[,2],prob=0.50)
Q50.a<-quantile(dbm[,2],prob=0.50)
   })
errmess<-geterrmessage() 
if (errmess!="" | !is.numeric(Q50.m) | !is.numeric(Q50.w) | !is.numeric(Q50.a)) {            
   errmess<-"error: false given columns of value"
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
LRL.m<-URL.m<-LRL.w<-URL.w<-LRL.a<-URL.a<-NA
if(is.na(RL1m) | is.na(RL2m) | !is.numeric(RL1m) | !is.numeric(RL2m)){
mess.m1<-"Note: evaluation of variability of median of data requires actual RLs used for male subjects.\n
         Program was run without evaluation of permissable uncertainty of median."
         sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
         cat("================================================================\n\n")
         cat(mess.m1, "\n")
         sink() 
         }else{
pU1<-pU(x=Q50.m,RL1=RL1m,RL2=RL2m,com=com+1)
LRL.m<-pU1[[1]]
URL.m<-pU1[[2]]
rm(pU1)
}
if(is.na(RL1w) | is.na(RL2w) | !is.numeric(RL1w) | !is.numeric(RL2w)){
mess.w1<-"Note: evaluation of variability of median of data requires actual RLs used for female subjects.\n
         Program was run without evaluation of permissable uncertainty of median."
         sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
         cat("================================================================\n\n")
         cat(mess.w1, "\n")
         sink() 
         }else{
pU1<-pU(x=Q50.w,RL1=RL1w,RL2=RL2w,com=com+1)
LRL.w<-pU1[[1]]
URL.w<-pU1[[2]]
rm(pU1)
}
if(is.na(RL1a) | is.na(RL2a) | !is.numeric(RL1a) | !is.numeric(RL2a)){
mess.a1<-"Note: evaluation of variability of median of data requires actual RLs used for all subjects.\n
         Program was run without evaluation of permissable uncertainty of median."
         sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
         cat("================================================================\n\n")
         cat(mess.a1, "\n")
         sink() 
         }else{
pU1<-pU(x=Q50.a,RL1=RL1a,RL2=RL2a,com=com+1)
LRL.a<-pU1[[1]]
URL.a<-pU1[[2]]
rm(pU1)
}
###############################################################
# Test of drift with GAM-function:
# male:
datem<-1:monm
datew<-1:monw
datea<-1:mon1
##############################
if(monm >=12 & !is.na(LRL.m) & !is.na(URL.m)){ 
# male:
try({
   dmbm1<-data.frame(Med=med.male,date=datem,number=n.male)
   y.H0<- gam(Med~1,weights=number,data=dmbm1)
   y.H1<-gam(Med~s(date),weights=as.double(n.male),data=dmbm1)
   xnew <- data.frame(date=seq(min(datem),max(datem),0.01),Med=1)
   fit <- predict(y.H1, newdata = xnew, , se = TRUE)$fit
   se <- predict( y.H1, newdata = xnew, se = TRUE)$se.fit
   lcl <- fit - 1.96 * se
   ymin<- min(fit - 2*1.96 * se)
   ymin<-min(ymin,med.male,2*LRL.m-Q50.m)
   ucl <- fit + 1.96 * se
   ymax<- max(fit + 2*1.96 * se)
   ymax<-max(ymax,med.male,2*URL.m-Q50.m)
   if (abs(ymin-ymax)< 1E-8)  {
      ymin<-ymin-10**(-com)
      ymax<-ymax+10**(-com)
      }
   pval.m<-anova(y.H1,y.H0,test='Chisq')$'Pr(>Chi)'[2]
   Bild1<-paste(result_file,"Months_m_med.",graph_format,sep="")
   Bild1.1<-paste(result_file1,"/","Months_m_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild1,width=7,height=5)
   plot(datem,med.male,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of the medians (male)",type="p",xaxt="n",pch=4,lwd=2)
   polygon(x=c(min(datem),max(datem),max(datem),min(datem)),
   y=c(URL.m,URL.m,LRL.m,LRL.m), col="gray87")
   abline(h=Q50.m,col="red",lty=1)
   abline(h=LRL.m,col="red",lty=2)
   abline(h=URL.m,col="red",lty=2)
   points(datem,med.male,col="black",pch=4,lwd=2)
   axis(1, at=datem,labels =labelm)
   lines(fit ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(lcl ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(ucl ~ xnew[,1],col="blue",lty=2,lwd=2)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall, red dashed lines (limits of grey region): " , "\n", 
                "permissable uncertainty of median overall, blue-dashed line: fitted smooth-curve of medians monthly," ,
                 "\n", "blue-dotted lines: CI of fitted smotth-curve"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild1)
   close_plot(graph_format=graph_format,file=Bild1.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: GAM-Test could not be used on the vlaues of male subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
   }else if(monm >=12 & (is.na(LRL.m) | is.na(URL.m))){ 
   try({
   dmbm1<-data.frame(Med=med.male,date=datem,number=n.male)
   y.H0<- gam(Med~1,weights=number,data=dmbm1)
   y.H1<-gam(Med~s(date),weights=as.double(n.male),data=dmbm1)
   xnew <- data.frame(date=seq(min(datem),max(datem),0.01),Med=1)
   fit <- predict(y.H1, newdata = xnew, , se = TRUE)$fit
   se <- predict( y.H1, newdata = xnew, se = TRUE)$se.fit
   lcl <- fit - 1.96 * se
   ymin<- min(fit - 2*1.96 * se)
   ymin<-min(ymin,med.male)
   ucl <- fit + 1.96 * se
   ymax<- max(fit + 2*1.96 * se)
   ymax<-max(ymax,med.male)
   if (abs(ymin-ymax)< 1E-8)  {
      ymin<-ymin-10**(-com)
      ymax<-ymax+10**(-com)
      }
   pval.m<-anova(y.H1,y.H0,test='Chisq')$'Pr(>Chi)'[2]
   Bild1<-paste(result_file,"Months_m_med.",graph_format,sep="")
   Bild1.1<-paste(result_file1,"/","Months_m_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild1,width=7,height=5)
   plot(datem,med.male,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (male)",type="p",xaxt="n",pch=4,lwd=2)
   abline(h=Q50.m,col="red",lty=1)
   points(datem,med.male,col="black",pch=4,lwd=2)
   axis(1, at=datem,labels =labelm)
   lines(fit ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(lcl ~ xnew[,1],col="blue",lty=3,lwd=2)
   lines(ucl ~ xnew[,1],col="blue",lty=3,lwd=2)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall," , "\n", 
                "blue-dashed line: fitted smooth-curve of medians monthly," ,
                 "\n", "blue-dotted lines: CI of fitted smotth-curve"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild1)
   close_plot(graph_format=graph_format,file=Bild1.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: GAM-Test could not be used on the vlaues of male subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###################
   }else if(monm < 12 & !is.na(LRL.m) & !is.na(URL.m)){ 
   try({ 
   ymin<-2*LRL.m-Q50.m
   ymax<-2*URL.m-Q50.m
   Bild1<-paste(result_file,"Months_m_med.",graph_format,sep="")
   Bild1.1<-paste(result_file1,"/","Months_m_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild1,width=7,height=5)
   plot(datem,med.male,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (male)",type="p",xaxt="n",pch=4,lwd=2)
   polygon(x=c(min(datem),max(datem),max(datem),min(datem)),
   y=c(URL.m,URL.m,LRL.m,LRL.m), col="gray87")
   abline(h=Q50.m,col="red",lty=1)
   abline(h=LRL.m,col="red",lty=2)
   abline(h=URL.m,col="red",lty=2)
   points(datem,med.male,col="black",pch=4,lwd=2)
   axis(1, at=datem,labels =labelm)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall, red dashed lines (limits of grey region): " , "\n", 
                "permissable uncertainty of median overall"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild1)
   close_plot(graph_format=graph_format,file=Bild1.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: Permissable uncertainty limits of median could not be calculated for male subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
   }else{
   Bild1<-paste(result_file,"Months_m_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild1,width=6,height=5)
   plot(1:10,1:10,main="",axes=FALSE,col="white",xlab="",ylab="")
   text(6,2,"No Test has been applied.",cex=2,col="red")
   abline(0,0)
   close_plot(graph_format=graph_format,file=Bild1)
} 
#################################
if(monw >=12 & !is.na(LRL.w) & !is.na(URL.w)){ 
# male:
try({
   dwbm1<-data.frame(Med=med.female,date=datew,number=n.female)
   y.H0<- gam(Med~1,weights=number,data=dwbm1)
   y.H1<-gam(Med~s(date),weights=as.double(n.female),data=dwbm1)
   xnew <- data.frame(date=seq(min(datew),max(datew),0.01),Med=1)
   fit <- predict(y.H1, newdata = xnew, , se = TRUE)$fit
   se <- predict( y.H1, newdata = xnew, se = TRUE)$se.fit
   lcl <- fit - 1.96 * se
   ymin<- min(fit - 2*1.96 * se)
   ymin<-min(ymin,med.female,2*LRL.w-Q50.w)
   ucl <- fit + 1.96 * se
   ymax<- max(fit + 2*1.96 * se)
   ymax<-max(ymax,med.female,2*URL.w-Q50.w)
   if (abs(ymin-ymax)< 1E-8)  {
      ymin<-ymin-10**(-com)
      ymax<-ymax+10**(-com)
      }
   pval.w<-anova(y.H1,y.H0,test='Chisq')$'Pr(>Chi)'[2]
   Bild2<-paste(result_file,"Months_w_med.",graph_format,sep="")
   Bild2.1<-paste(result_file1,"/","Months_w_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild2,width=7,height=5)
   plot(datew,med.female,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of the medians (female)",type="p",xaxt="n",pch=4,lwd=2)
   polygon(x=c(min(datew),max(datew),max(datew),min(datew)),
   y=c(URL.w,URL.w,LRL.w,LRL.w), col="gray87")
   abline(h=Q50.w,col="red",lty=1)
   abline(h=LRL.w,col="red",lty=2)
   abline(h=URL.w,col="red",lty=2)
   points(datew,med.female,col="black",pch=4,lwd=2)
   axis(1, at=datew,labels =labelw)
   lines(fit ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(lcl ~ xnew[,1],col="blue",lty=3,lwd=2)
   lines(ucl ~ xnew[,1],col="blue",lty=3,lwd=2)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall, red dashed lines (limits of grey region): " , "\n", 
                "permissable uncertainty of median overall, blue-dashed line: fitted smooth-curve of medians monthly," ,
                 "\n", "blue-dotted lines: CI of fitted smotth-curve"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild2)
   close_plot(graph_format=graph_format,file=Bild2.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: GAM-Test could not be used on the vlaues of female subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
   }else if(monw >=12 & (is.na(LRL.w) | is.na(URL.w))){ 
   try({
   dwbm1<-data.frame(Med=med.female,date=datew,number=n.female)
   y.H0<- gam(Med~1,weights=number,data=dwbm1)
   y.H1<-gam(Med~s(date),weights=as.double(n.female),data=dwbm1)
   xnew <- data.frame(date=seq(min(datew),max(datew),0.01),Med=1)
   fit <- predict(y.H1, newdata = xnew, , se = TRUE)$fit
   se <- predict( y.H1, newdata = xnew, se = TRUE)$se.fit
   lcl <- fit - 1.96 * se
   ymin<- min(fit - 2*1.96 * se)
   ymin<-min(ymin,med.female)
   ucl <- fit + 1.96 * se
   ymax<- max(fit + 2*1.96 * se)
   ymax<-max(ymax,med.female)
   if (abs(ymin-ymax)< 1E-8)  {
      ymin<-ymin-10**(-com)
      ymax<-ymax+10**(-com)
      }
   pval.w<-anova(y.H1,y.H0,test='Chisq')$'Pr(>Chi)'[2]
   Bild2<-paste(result_file,"Months_w_med.",graph_format,sep="")
   Bild2.1<-paste(result_file1,"/","Months_w_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild2,width=7,height=5)
   plot(datew,med.female,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (female)",type="p",xaxt="n",pch=4,lwd=2)
   abline(h=Q50.w,col="red",lty=1)
   points(datew,med.female,col="black",pch=4,lwd=2)
   axis(1, at=datew,labels =labelw)
   lines(fit ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(lcl ~ xnew[,1],col="blue",lty=3,lwd=2)
   lines(ucl ~ xnew[,1],col="blue",lty=3,lwd=2)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall," , "\n", 
                "blue-dashed line: fitted smooth-curve of medians monthly," ,
                 "\n", "blue-dotted lines: CI of fitted smotth-curve"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild2)
   close_plot(graph_format=graph_format,file=Bild2.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: GAM-Test could not be used on the vlaues of female subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###################
   }else if(monw < 12 & !is.na(LRL.w) & !is.na(URL.w)){ 
   try({ 
   ymin<-2*LRL.w-Q50.w
   ymax<-2*URL.w-Q50.w
   Bild2<-paste(result_file,"Months_w_med.",graph_format,sep="")
   Bild2.1<-paste(result_file1,"/","Months_w_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild2,width=7,height=5)
   plot(datew,med.female,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (female)",type="p",xaxt="n",pch=4,lwd=2)
   polygon(x=c(min(datew),max(datew),max(datew),min(datew)),
   y=c(URL.w,URL.w,LRL.w,LRL.w), col="gray87")
   abline(h=Q50.w,col="red",lty=1)
   abline(h=LRL.w,col="red",lty=2)
   abline(h=URL.w,col="red",lty=2)
   points(datew,med.female,col="black",pch=4,lwd=2)
   axis(1, at=datew,labels =labelw)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall, red dashed lines (limits of grey region): " , "\n", 
                "permissable uncertainty of median overall"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild2)
   close_plot(graph_format=graph_format,file=Bild2.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: Permissable uncertainty limits of median could not be calculated for female subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
   }else{
   Bild2<-paste(result_file,"Months_w_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild2,width=6,height=5)
   plot(1:10,1:10,main="",axes=FALSE,col="white",xlab="",ylab="")
   text(6,2,"No Test has been applied.",cex=2,col="red")
   abline(0,0)
   close_plot(graph_format=graph_format,file=Bild2)
} 
#################################
if(mon1 >=12 & !is.na(LRL.a) & !is.na(URL.a)){ 
# all:
try({
   dbm1<-data.frame(Med=med.all,date=datea,number=n.all)
   y.H0<- gam(Med~1,weights=number,data=dbm1)
   y.H1<-gam(Med~s(date),weights=as.double(n.all),data=dbm1)
   xnew <- data.frame(date=seq(min(datea),max(datea),0.01),Med=1)
   fit <- predict(y.H1, newdata = xnew, , se = TRUE)$fit
   se <- predict( y.H1, newdata = xnew, se = TRUE)$se.fit
   lcl <- fit - 1.96 * se
   ymin<- min(fit - 2*1.96 * se)
   ymin<-min(ymin,med.all,2*LRL.a-Q50.a)
   ucl <- fit + 1.96 * se
   ymax<- max(fit + 2*1.96 * se)
   ymax<-max(ymax,med.all,2*URL.a-Q50.a)
   if (abs(ymin-ymax)< 1E-8)  {
      ymin<-ymin-10**(-com)
      ymax<-ymax+10**(-com)
      }
   pval.a<-anova(y.H1,y.H0,test='Chisq')$'Pr(>Chi)'[2]
   Bild3<-paste(result_file,"Months_a_med.",graph_format,sep="")
   Bild3.1<-paste(result_file1,"/","Months_a_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild3,width=7,height=5)
   plot(datea,med.all,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (all subjects)",type="p",xaxt="n",pch=4,lwd=2)
   polygon(x=c(min(datea),max(datea),max(datea),min(datea)),
   y=c(URL.a,URL.a,LRL.a,LRL.a), col="gray87")
   abline(h=Q50.a,col="red",lty=1)
   abline(h=LRL.a,col="red",lty=2)
   abline(h=URL.a,col="red",lty=2)
   points(datea,med.all,col="black",pch=4,lwd=2)
   axis(1, at=datea,labels =label1)
   lines(fit ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(lcl ~ xnew[,1],col="blue",lty=3,lwd=2)
   lines(ucl ~ xnew[,1],col="blue",lty=3,lwd=2)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall, red dashed lines (limits of grey region): " , "\n", 
                "permissable uncertainty of median overall, blue-dashed line: fitted smooth-curve of medians monthly," ,
                 "\n", "blue-dotted lines: CI of fitted smotth-curve"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild3)
   close_plot(graph_format=graph_format,file=Bild3.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: GAM-Test could not be used on the vlaues of all subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
   }else if(mon1 >=12 & (is.na(LRL.a) | is.na(URL.a))){ 
   try({
   dbm1<-data.frame(Med=med.all,date=datea,number=n.all)
   y.H0<- gam(Med~1,weights=number,data=dbm1)
   y.H1<-gam(Med~s(date),weights=as.double(n.all),data=dbm1)
   xnew <- data.frame(date=seq(min(datea),max(datea),0.01),Med=1)
   fit <- predict(y.H1, newdata = xnew, , se = TRUE)$fit
   se <- predict( y.H1, newdata = xnew, se = TRUE)$se.fit
   lcl <- fit - 1.96 * se
   ymin<- min(fit - 2*1.96 * se)
   ymin<-min(ymin,med.all)
   ucl <- fit + 1.96 * se
   ymax<- max(fit + 2*1.96 * se)
   ymax<-max(ymax,med.all)
   if (abs(ymin-ymax)< 1E-8)  {
      ymin<-ymin-10**(-com)
      ymax<-ymax+10**(-com)
      }
   pval.a<-anova(y.H1,y.H0,test='Chisq')$'Pr(>Chi)'[2]
   Bild3<-paste(result_file,"Months_a_med.",graph_format,sep="")
   Bild3.1<-paste(result_file1,"/","Months_a_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild3,width=7,height=5)
   plot(datea,med.all,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (all subjects)",type="p",xaxt="n",pch=4,lwd=2)
   abline(h=Q50.a,col="red",lty=1)
   points(datea,med.all,col="black",pch=4,lwd=2)
   axis(1, at=datea,labels =label1)
   lines(fit ~ xnew[,1],col="blue",lty=2,lwd=2)
   lines(lcl ~ xnew[,1],col="blue",lty=3,lwd=2)
   lines(ucl ~ xnew[,1],col="blue",lty=3,lwd=2)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall," , "\n", 
                "blue-dashed line: fitted smooth-curve of medians monthly," ,
                 "\n", "blue-dotted lines: CI of fitted smotth-curve"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild3)
   close_plot(graph_format=graph_format,file=Bild3.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: GAM-Test could not be used on the vlaues of all subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###################
   }else if(mon1 < 12 & !is.na(LRL.a) & !is.na(URL.a)){ 
   try({ 
   ymin<-2*LRL.a-Q50.a
   ymax<-2*URL.a-Q50.a
   Bild3<-paste(result_file,"Months_a_med.",graph_format,sep="")
   Bild3.1<-paste(result_file1,"/","Months_a_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild3,width=7,height=5)
   plot(datea,med.all,col="black",ylim=c(ymin,ymax),xlab="",ylab=paste(testvalue," ",testunit),
   main="Time plot of medians (all subjects)",type="p",xaxt="n",pch=4,lwd=2)
   polygon(x=c(min(datea),max(datea),max(datea),min(datea)),
   y=c(URL.a,URL.a,LRL.a,LRL.a), col="gray87")
   abline(h=Q50.a,col="red",lty=1)
   abline(h=LRL.a,col="red",lty=2)
   abline(h=URL.a,col="red",lty=2)
   points(datew,med.female,col="black",pch=4,lwd=2)
   axis(1, at=datea,labels =label1)
   abline(0,0)
   mtext(paste("black cross points: medians monthly, red solid line: median overall, red dashed lines (limits of grey region): " , "\n", 
                "permissable uncertainty of median overall"),
                   cex=0.8,font=3,outer=F,line=4,side=1)
   close_plot(graph_format=graph_format,file=Bild3)
   close_plot(graph_format=graph_format,file=Bild3.1)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: Permissable uncertainty limits of median could not be calculated for all subjects."
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
   }else{
   Bild3<-paste(result_file,"Months_a_med.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=Bild3,width=6,height=5)
   plot(1:10,1:10,main="",axes=FALSE,col="white",xlab="",ylab="")
   text(6,2,"No Test has been applied.",cex=2,col="red")
   abline(0,0)
   close_plot(graph_format=graph_format,file=Bild3)
} 
###############################################################
###                                                         ###
###     Generating the results table                        ###
###                                                         ###
###############################################################
###############################################################
#
cat("program is generating results table ....\n")
#
names.n<-c("Number: Male","Number: Female","Number: All")
names05<-c("0.05-Quantile: Male","0.05-Quantile: Female","0.05-Quantile: All")
names25<-c("0.25-Quantile: Male","0.25-Quantile: Female","0.25-Quantile: All")
names50<-c("0.50-Quantile: Male","0.50-Quantile: Female","0.50-Quantile: All")
names75<-c("0.75-Quantile: Male","0.75-Quantile: Female","0.75-Quantile: All")
names95<-c("0.95-Quantile: Male","0.95-Quantile: Female","0.95-Quantile: All")
rnames<-c(names.n,names05,names25,names50,names75,names95)
##############################################################
# Function mmL is used for the case that in some months only
# male or female subjects are treated in the clinic!
##############################################################
mmL <- function(v,mx) {
   mx[] <- NA
	 mx[sapply(names(v),function(x) which(names(mx)==x))] <- v
	 mx
   }
q05m<-mmL(q05m,q50)
q05m<-round(q05m,com+1)
q05w<-mmL(q05w,q50)
q05w<-round(q05w,com+1)
q25m<-mmL(q25m,q50)
q25m<-round(q25m,com+1)
q25w<-mmL(q25w,q50)
q25w<-round(q25w,com+1)
q50m<-mmL(q50m,q50)
q50m<-round(q50m,com+1)
q50w<-mmL(q50w,q50)
q50w<-round(q50w,com+1)
q75m<-mmL(q75m,q50)
q75m<-round(q75m,com+1)
q75w<-mmL(q75w,q50)
q75w<-round(q75w,com+1)
q95m<-mmL(q95m,q50)
q95m<-round(q95m,com+1)
q95w<-mmL(q95w,q50)
q95w<-round(q95w,com+1) 
try({                
   d<-rbind(n.male,n.female,n.all,q05m,q05w,q05,q25m,q25w,q25,q50m,q50w,q50,q75m,q75w,q75,q95m,q95w,q95)
   d<-as.data.frame(d,row.names=rnames)
   write.table(d, file = flag_file,quote=F,sep=";",col.names=TRUE)
   write.csv(d, file = savename1a,quote=F)
   Result1a<-paste(result_file1,"/","result_drift.csv",sep="")
   write.csv(d, file = Result1a,quote=F)
   })
errmess<-geterrmessage() 
if (errmess!="") {            
   errmess<-"error: step-1a has not been carried out. Control if the data set meets required informations."
   } else {
   errmess<-""
   }
#################################
sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
end.time <- date()
cat("Program start:", start.time,"\n")
cat("program end:  ", end.time,"\n")
cat("================================================================\n\n")
cat(errmess, "\n")
sink()
write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
cat("================================================================\n\n")
cat("R_program1 finished\n\n")
cat("================================================================\n")
sink()
quit(save = "no", status = 0, runLast = FALSE)
